var Promise = require('bluebird');
var MRAA = require('mraa');

var BUTTON_MULTI_CLICK_DELAY = 750;

function SimpleButton(name, pin) {
	this.name = name;
	this.pin = pin;
	this.btn = null;
	this.lastClicked = 0;
}

/**
 * @param args {
 *   onClick: event handler for click event
 *   thisObj: 'this' object to set to; defaults to this simple button instance
 * }
 */
SimpleButton.prototype.init = function(args) {
	console.log('button ' + this.name + ' init');
	this.btn = new MRAA.Gpio(this.pin);
	this.btn.dir(MRAA.DIR_IN);

	this.btn.isr(MRAA.EDGE_FALLING, SimpleButton.prototype.onClick.bind(this, args.onClick, args.thisObj || this ));
}

SimpleButton.prototype.selfTestAsync = function() {
	console.log('button ' + this.name + ' self test');
	return Promise.resolve(true);
}

SimpleButton.prototype.start = function() {
	// Nothing to do
}

SimpleButton.prototype.onClick = function(callback, thisObj) {
	var lastLastClicked = this.lastClicked;
	this.lastClicked = Date.now();
	if ((this.lastClicked - lastLastClicked) >= BUTTON_MULTI_CLICK_DELAY) {
		callback.call(thisObj);
	}
}

module.exports = SimpleButton;
